package dwatch;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import dwatch.digitalwatch.DigitalwatchStatemachine;
import dwatch.digitalwatch.IDigitalwatchStatemachine.SCIButtons;
import dwatch.digitalwatch.IDigitalwatchStatemachine.SCIDisplayOperationCallback;

public class DigitalWatchViewImpl implements MouseListener, SCIDisplayOperationCallback {
	private JPanel displayPanel;
	private JLabel timeLabel;
	private JLabel dateLabel;
	private JLabel alarmIcon;
	
	private SCIButtons listener;
	private DigitalWatchController model;
	private DigitalwatchStatemachine sm;
    
	public DigitalWatchViewImpl(DigitalWatchController model, SCIButtons listener, DigitalwatchStatemachine sm) {
		this.model = model;
		this.listener = listener;
		this.sm = sm;
		initializeGUI();
	}
    
	@Override
	public void refreshTimeDisplay() {
		timeLabel.setText(model.getTime());
		dateLabel.setText(model.getDate());
	}
	
	@Override
	public void refreshChronoDisplay() {
		timeLabel.setText(model.getChrono());
	}
    
    
	@Override
	public void hidePos(long pos) {
		timeLabel.setText(model.getTimeLabelAsForHiding((int)pos));
		dateLabel.setText(model.getDateLabelAsForHiding((int)pos));
	}
	
	@Override
	public void setIndiglo() {
		displayPanel.setBackground(Color.CYAN);
	}
	
	@Override
	public void unsetIndiglo() {
		displayPanel.setBackground(Color.WHITE);
	}
	
	private void initializeGUI() {
		JLayeredPane lpane = new JLayeredPane();
		final Image image = new ImageIcon(getClass().getResource("/watch.gif")).getImage();
		@SuppressWarnings("serial")
		JPanel imagePanel = new JPanel() {
			public void paintComponent(Graphics g) { g.drawImage(image, 0, 0, null); }
		};
		Dimension size = new Dimension(image.getWidth(null), image.getHeight(null)+22);
		imagePanel.setPreferredSize(size);
		imagePanel.setMinimumSize(size);
		imagePanel.setMaximumSize(size);
		imagePanel.setSize(size);
        imagePanel.setLayout(null);
        
		imagePanel.addMouseListener(this);
		
		displayPanel = new JPanel();
		displayPanel.setBounds(52,97,122,54);
		displayPanel.setBackground(Color.WHITE);
		displayPanel.setForeground(Color.BLACK);
        
		dateLabel = new JLabel();
		dateLabel.setBounds(94,100,60,20);
		dateLabel.setFont(new Font("Courier", Font.PLAIN, 12));
        
		timeLabel = new JLabel();
		timeLabel.setBounds(84,120,80,20);
		timeLabel.setFont(new Font("Courier", Font.PLAIN, 14));
		
		alarmIcon = new JLabel();
		alarmIcon.setBounds(154,97,20,20);
		alarmIcon.setVisible(true);
		
		lpane.add(imagePanel, 0, 0);
		lpane.add(displayPanel, 1, 0);
		lpane.add(dateLabel, 3, 0);
		lpane.add(timeLabel, 3, 0);
		lpane.add(alarmIcon, 3, 0);
		
		JFrame frame = new JFrame();
	    frame.getContentPane().add(lpane);
	    frame.setPreferredSize(imagePanel.getPreferredSize());
	    frame.setResizable(false);
	    frame.pack();
	    frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
	    frame.setVisible(true);
	}
    
	// ==========================================================
	// Implementation of MouseEventListener interface
	// ==========================================================
	Rectangle[] buttonBounds = {
    new Rectangle(4, 60, 10, 11),
    new Rectangle(209, 60, 10, 11),
    new Rectangle(4, 162, 10, 11),
    new Rectangle(213, 162, 10, 11)
    };
	String[] buttonPressedEvents = {
    "topLeftPressed", "topRightPressed",
    "bottomLeftPressed", "bottomRightPressed"
    };
	String[] buttonReleasedEvents = {
    "topLeftReleased", "topRightReleased",
    "bottomLeftReleased", "bottomRightReleased"
    };
    
	@Override
	public void mousePressed(MouseEvent e) {
		Point point = e.getPoint();
		for (int i = 0; i < buttonBounds.length; i++) {
			if (buttonBounds[i].contains(point)) {
				notifyEvent(buttonPressedEvents[i]);
				break;
			}
		}
	}
    
	@Override
	public void mouseReleased(MouseEvent e) {
		Point point = e.getPoint();
		for (int i = 0; i < buttonBounds.length; i++) {
			if (buttonBounds[i].contains(point)) {
				notifyEvent(buttonReleasedEvents[i]);
				break;
			}
		}
	}
    
	@Override
	public void mouseClicked(MouseEvent e) {}
	@Override
	public void mouseEntered(MouseEvent e) {}
	@Override
	public void mouseExited(MouseEvent e) {}
    
	// ==========================================================
	// Management of Event listeners (Interface with the State Machine)
	// ==========================================================
	private void notifyEvent(String event) {
		if (event.equals("topLeftPressed"))
			listener.raiseTopLeftPressed();
		else if (event.equals("topLeftReleased"))
			listener.raiseTopLeftReleased();
		else if (event.equals("topRightPressed"))
			listener.raiseTopRightPressed();
		else if (event.equals("topRightReleased"))
			listener.raiseTopRightReleased();
		else if (event.equals("bottomLeftPressed"))
			listener.raiseBotLeftPressed();
		else if (event.equals("bottomLeftReleased"))
			listener.raiseBotLeftReleased();
		else if (event.equals("bottomRightPressed"))
			listener.raiseBotRightPressed();
		else if (event.equals("bottomRightReleased"))
			listener.raiseBotRightReleased();
		sm.runCycle();
	}
}
